@@ -1,5 +1,5 @@
 package HTTP::API::Client;
-$HTTP::API::Client::VERSION = '0.02';
+$HTTP::API::Client::VERSION = '0.03';
 use strict;
 use warnings;
 
@@ -30,30 +30,67 @@ Get Json Data - grab the content body from the response and json decode
 
 At the moment, only support query string and json data in and out
 
+=head1 ENVIRONMENT VARIABLES
+
+These enviornment variables expose the control without changing the existing code.
+
+HTTP VARIABLES
+
+ HTTP_USERNAME   - basic auth username
+ HTTP_PASSWORD   - basic auth password
+ HTTP_AUTH_TOKEN - basic auth token string
+ HTTP_CHARSET    - content type charset. default utf8
+ HTTP_TIMEOUT    - timeout the request for ??? seconds. default 60 seconds.
+ SSL_VERIFY      - verify ssl url. default is off
+
+DEBUG VARIABLES
+
+ DEBUG_IN_OUT               - print out request and response in string to STDERR
+ DEBUG_SEND_OUT             - print out request in string to STDERR
+ DEBUG_RESPONSE             - print out response in string to STDERR
+ DEBUG_RESPONSE_HEADER_ONLY - print out response header only without the body
+ DEBUG_RESPONSE_IF_FAIL     - only print out response in string if fail.
+
+RETRY VARIABLES
+
+ RETRY_FAIL_RESPONSE  - number of time to retry if resposne comes back is failed. default 0 retry
+ RETRY_FAIL_STATUS    - only retry if specified status code. e.g. 500,404
+ RETRY_DELAY          - retry with wait time of ??? seconds in between
+
 =cut
 
 use URI;
 use Mouse;
+use JSON::XS;
 use Try::Tiny;
 use HTTP::Request;
 use HTTP::Headers;
 use LWP::UserAgent;
 
 has username => (
-    is  => "rw",
-    isa => "Str",
+    is         => "rw",
+    isa        => "Str",
+    lazy_build => 1,
 );
 
+sub _build_username { $ENV{HTTP_USERNAME} || qq{} }
+
 has password => (
-    is  => "rw",
-    isa => "Str",
+    is         => "rw",
+    isa        => "Str",
+    lazy_build => 1,
 );
 
+sub _build_password { $ENV{HTTP_PASSWORD} || qq{} }
+
 has auth_token => (
-    is  => "rw",
-    isa => "Str",
+    is         => "rw",
+    isa        => "Str",
+    lazy_build => 1,
 );
 
+sub _build_auth_token { $ENV{HTTP_AUTH_TOKEN} || qq{} }
+
 has base_url => (
     is  => "rw",
     isa => "URI",
@@ -65,53 +102,136 @@ has last_response => (
 );
 
 has charset => (
-    is      => "rw",
-    isa     => "Str",
-    default => "utf8",
+    is         => "rw",
+    isa        => "Str",
+    lazy_build => 1,
 );
 
+sub _build_charset { $ENV{HTTP_CHARSET} || "utf8" }
+
 has browser_id => (
     is         => "rw",
     isa        => "Str",
     lazy_build => 1,
 );
 
+sub _build_browser_id {
+    my $self = shift;
+    my $ver = $HTTP::API::Client::VERSION || -1;
+    return "HTTP API Client v$ver";
+}
+
 has content_type => (
     is         => "rw",
     isa        => "Str",
     lazy_build => 1,
 );
 
+sub _build_content_type {
+    my $self    = shift;
+    my $charset = $self->charset;
+    return "application/json; charset=$charset";
+}
+
 has ua => (
     is         => "rw",
     isa        => "LWP::UserAgent",
     lazy_build => 1,
 );
 
+sub _build_ua {
+    my $self       = shift;
+    my $ssl_verify = $self->ssl_verify;
+    my $ua =
+      LWP::UserAgent->new( ssl_opts => { verify_hostname => $ssl_verify } );
+    $ua->agent( $self->browser_id );
+    $ua->timeout( $self->timeout );
+    return $ua;
+}
+
 has ssl_verify => (
     is         => "rw",
     isa        => "Bool",
     lazy_build => 1,
 );
 
+sub _build_ssl_verify {
+    return $ENV{SSL_VERIFY} // 0;
+}
+
 has retry => (
     is         => "rw",
     isa        => "HashRef",
     lazy_build => 1,
 );
 
-has timeout => (
-    is      => "rw",
-    isa     => "Int",
-    default => 60,
-);
+sub _build_retry {
+    my $self   = shift;
+    my %retry  = %{ $self->retry_config || {} };
+    my $count  = $retry{fail_response};
+    my %status = map { $_ => 1 } split /,/, $retry{fail_status};
 
-has json => (
-    is => "rw",
-    isa => "JSON::XS",
+    my $delay = $retry{delay};
+
+    return {
+        count  => $count,
+        status => \%status,
+        delay  => $delay,
+    };
+}
+
+has retry_config => (
+    is         => "rw",
+    isa        => "HashRef",
     lazy_build => 1,
 );
 
+sub _build_retry_config {
+    return {
+        fail_response => $ENV{RETRY_FAIL_RESPONSE} // 0,
+        fail_status => $ENV{RETRY_FAIL_STATUS} || q{},
+        delay => $ENV{RETRY_DELAY} // 5,
+    };
+}
+
+has timeout => (
+    is         => "rw",
+    isa        => "Int",
+    lazy_build => 1,
+);
+
+sub _build_timeout { return $ENV{HTTP_TIMEOUT} || 60 }
+
+has json => (
+    is         => "rw",
+    isa        => "JSON::XS",
+    lazy_build => 1,
+);
+
+sub _build_json {
+    my $self    = shift;
+    my $json    = JSON::XS->new->canonical(1);
+    my $charset = $self->charset;
+    eval { $json->$charset };
+    return $json;
+}
+
+has debug_flags => (
+    is         => "rw",
+    isa        => "HashRef",
+    lazy_build => 1,
+);
+
+sub _build_debug_flags {
+    return {
+        in_out               => $ENV{DEBUG_IN_OUT},
+        send_out             => $ENV{DEBUG_SEND_OUT},
+        response             => $ENV{DEBUG_RESPONSE},
+        response_header_only => $ENV{DEBUG_RESPONSE_HEADER_ONLY},
+        response_if_fail     => $ENV{DEBUG_RESPONSE_IF_FAIL},
+    };
+}
+
 no Mouse;
 
 sub get {
@@ -152,6 +272,8 @@ sub send {
     my $retry_count  = $self->retry->{count} // 1;
     my %retry_status = %{ $self->retry->{status} || {} };
     my $retry_delay  = $self->retry->{delay} // 5;
+    my %debug        = %{ $self->debug_flags || {} };
+    $DB::single=2;
 
     my $response;
 
@@ -161,7 +283,7 @@ sub send {
 
         $response = $ua->request($req);
 
-        if ( $ENV{DEBUG_IN_OUT} || $ENV{DEBUG_SEND_OUT} ) {
+        if ( $debug{in_out} || $debug{send_out} ) {
             print STDERR "-- REQUEST --\n";
             if ( $retry_count && $retry ) {
                 print STDERR "-- RETRY $retry of $retry_count\n";
@@ -170,17 +292,17 @@ sub send {
             print STDERR "\n";
         }
 
-        my $debug_response = $ENV{DEBUG_IN_OUT} || $ENV{DEBUG_RESPONSE};
+        my $debug_response = $debug{in_out} || $debug{response};
 
         $debug_response = 0
-          if $ENV{DEBUG_RESPONSE_IF_FAIL} && $response->is_success;
+          if $debug{response_if_fail} && $response->is_success;
 
         if ($debug_response) {
             my $used_time = time - $started_time;
 
             print STDERR "-- RESPONSE $used_time sec(s) --\n";
 
-            print STDERR $ENV{DEBUG_RESPONSE_HEADER_ONLY}
+            print STDERR $debug{response_header_only}
               ? $response->headers->as_string
               : $response->as_string;
 
@@ -210,6 +332,9 @@ sub send {
     }
 
     return $self->last_response($response);
+## Please see file perltidy.ERR
+## Please see file perltidy.ERR
+## Please see file perltidy.ERR
 }
 
 sub json_response {
@@ -323,55 +448,4 @@ sub _basic_authenticator {
     $req->headers->authorization_basic( $username, $password );
 }
 
-sub _build_content_type {
-    my $self    = shift;
-    my $charset = $self->charset;
-    return "application/json; charset=$charset";
-}
-
-sub _build_ua {
-    my $self       = shift;
-    my $ssl_verify = $self->ssl_verify;
-    my $ua =
-      LWP::UserAgent->new( ssl_opts => { verify_hostname => $ssl_verify } );
-    $ua->agent( $self->browser_id );
-    $ua->timeout( $self->timeout );
-    return $ua;
-}
-
-sub _build_browser_id {
-    my $self = shift;
-    my $ver = $HTTP::API::Client::VERSION || -1;
-    return "Monitoring API Client v$ver";
-}
-
-sub _build_ssl_verify {
-    return $ENV{SSL_VERIFY} // 0;
-}
-
-sub _build_retry {
-    my $count = $ENV{RETRY_FAIL_RESPONSE} // 0;
-    my $status =
-      $ENV{RETRY_FAIL_STATUS}
-      ? eval { $ENV{RETRY_FAIL_STATUS} }
-      : {};
-
-    if ( my $error = $@ ) {
-        die "RETRY_FAIL_STATUS error: $error";
-    }
-
-    my $delay = $ENV{RETRY_DELAY} // 5;
-
-    return {
-        count  => $count,
-        status => $status,
-        delay  => $delay,
-    };
-}
-
-sub _build_json {
-    require JSON::XS;
-    return JSON::XS->new->canonical(1)->utf8;
-}
-
 1;
